;
; MVCOMP depacking routine in 8086 assembly
; Copyright (C) 2024 Mateusz Viste, MIT license
;
; http://mvcomp.sourceforge.net
;
; compatible with nasm, wasm, etc
; assembles to some 58 bytes of 8086 machine code.
;
;
; parameters:
; ES:SI  the source buffer (start of MVCOMP compressed stream)
; ES:DI  the dst buffer (may overlap the source buffer, see IN-PLACE.TXT)
; BX     the length, in bytes, of the compressed stream
;
; On exit, DI points one byte past the last uncompressed byte.
; Registers AX, BX, CX, DX and SI are destroyed
;

    push ds
    shr bx, 1          ; convert byte length to number of words
    cld                ; make sure stosw and friends move forward

    ; set ds = es = bufseg
    push es
    pop ds

    xor dx, dx         ; literal continuation counter

AGAIN:

    ; do I have any input?
    test bx, bx
    jz KONIEC
    dec bx

     ; load token
    lodsw              ; mov ax, [ds:si] + inc si + inc si

    ; literal continuation?
    test dx, dx
    jz TRY_BACKREF
    stosw
    dec dx             ; a byte shorter than dec dl
    jmp AGAIN

    ; back ref?
    ; token format: LLLL OOOO OOOO OOOO
TRY_BACKREF:
    test ax, 0xf000
    jz LITERAL_START   ; else it's a literal start
    ; save reg
    push si
    ; copy (ah>>4)+1 bytes from (ax & 0x0FFF)+1
    ; prep DS:SI = source ; ES:DI = destination ; CX = len
    mov ch, ah         ; all this drama is about setting CX to the high
    mov cl, 4          ; nibble of AX using code as compact as possible.
    shr ch, cl
    mov cl, ch
    xor ch, ch
    inc cx
    and ax, 0x0fff     ; clear the backref length bits, keep only the offset
    inc ax
    mov si, di
    sub si, ax
    ; do the copy
    rep movsb          ; copy cx bytes from ds:si to es:di + inc si + inc di

    ; restore reg
    pop si
    jmp AGAIN

    ; token format: 0000 UUUU  BBBB BBBB
LITERAL_START:         ; write al to dst and set literal counter
    stosb              ; mov [es:di], al + inc di
    mov dl, ah         ; ah high nibble is guaranteed to be zero
    jmp AGAIN

    KONIEC:
    pop ds
