#!/bin/bash
#
# simple test suite for mvcomp
# Mateusz Viste, 2024

#
# LET'S PERFORM SOME COMPRESSION JOBS
#

rm -f *.mv

./mvcomp ../testdata/fdiskpl.txt fdiskpl.mv
./mvcomp ../testdata/kvazar.jpg kvazar.mv
./mvcomp ../testdata/przasn.mp3 przasn.mv
./mvcomp ../testdata/rand.bin rand.mv
./mvcomp ../testdata/romjul.txt romjul.mv
./mvcomp ../testdata/x86.bin x86.mv
./mvcomp ../testdata/zero.bin zero.mv

cat ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt ../testdata/romjul.txt > big.txt
./mvcomp big.txt big.mv
rm big.txt

#
# NOW UNCOMPRESS AND LOOK FOR DATA CORRUPTION
#

ERRFLAG="GOOD"

function chkfil {
FSIZ=$(stat -c "%s" $1.mv)
if [ $FSIZ -gt $3 ] ; then
echo "WARNING! $1.mv compressed to less than expected ($FSIZ bytes, expected $3)"
ERRFLAG="BAD"
fi
./mvucomp $1.mv $1.$2 > /dev/null
diff -q ../testdata/$1.$2 $1.$2
if [ ! $? -eq 0 ] ; then ERRFLAG="BAD" ; fi


rm ./$1.$2
}

#      file     original ext    expected compression
chkfil fdiskpl  txt               8238
chkfil kvazar   jpg             220152
chkfil przasn   mp3             195156
chkfil rand     bin             270554
chkfil romjul   txt              78734
chkfil x86      bin              54580
chkfil zero     bin              32770

echo ''
if [ "$ERRFLAG" == "GOOD" ] ; then
echo 'all good :-)'
rm *.mv
else
echo 'ERROR: INCONSISTENT RESULTS, CHECK LOGS!'
exit 1
fi

exit 0
